// escrow-sign-tx.js
// Sign the escrow transaction with a private key

var bitcoin = require('bitcoinjs-lib');
var prompt = require('cli-prompt');
var fs = require('fs')

// convert 'satoshi' to bitcoin value
var satoshiToBTC = function(value) {
	return value * 0.00000001;
}

// load the escrow json file
var escrow_filename = process.argv[2];
if (!escrow_filename) {
	console.log("Please provide the escrow filename.")
	console.log("example usage: node escrow-sign-tx.js ./escrow-3Bh334.json")
	return;	
}
var escrow = require(escrow_filename);

// report the details of the escrow transaction
console.log("Sign withdraw of", satoshiToBTC(escrow.amount), "BTC to", escrow.pay_to_address);
console.log("Transaction has", escrow.signatures, "of", escrow.signatures_required, "signatures");

// exit if escrow has been signed by two signatures
if (escrow.signatures >= escrow.signatures_required) {
	console.log("Escrow transaction is ready to broadcast.");
	return;
}

prompt('Private key for signing the transaction (WIF format): ', function (val) {	                          
	try {
		// create the transaction builder object
		var script = bitcoin.Script.fromHex(escrow.redeem_script);
		var tx = bitcoin.Transaction.fromHex(escrow.encoded_tx);
		var txb = bitcoin.TransactionBuilder.fromTransaction(tx);

		// sign the transaction
		var private_key = bitcoin.ECKey.fromWIF(val);
		txb.sign(0, private_key, script); 
		
		// build the transaction, either first or second signature
		if (escrow.signatures == 0) {
			// record the first signature
			escrow.encoded_tx = txb.buildIncomplete().toHex();
			escrow.signatures = 1
			console.log("Escrow transaction requires one more signature.");
		}
		else { 
			// record the second signature
			var tx = txb.build();
			escrow.encoded_tx = tx.toHex();
			escrow.signatures = 2;
			escrow.tx_hash = tx.getId();
			console.log("Escrow transaction is ready to broadcast.");
		}
		
		// write the signed transaction to disk
	  fs.writeFile(escrow_filename, JSON.stringify(escrow, null, 4), function(err) {
    		if (err) { return console.log(err) }
		});

	} catch(e) {
		console.log("Unable to sign:", e.message);
	}
})