// send-transaction.js 
// Create and send a Bitcoin transaction
// 1 - query for unspent outputs
// 2 - create a transaction
// 3 - forward the transaction to the network

var bitcoin = require('bitcoinjs-lib');
var request = require('request');
var prompt = require('cli-prompt');

// convert 'satoshi' to bitcoin value
var satoshiToBTC = function(value) {
	return value * 0.00000001;
};

// broadcasts a transaction to the network via blockchain.info
var broadcast_tx = function(tx) {
	var options = {
		url: "https://blockchain.info/pushtx", 
		formData: { 
			tx: tx.toHex() 
		}
	};

	request.post(options, function(err, httpResponse, body) {
		if (err) {
			console.error('Request failed:', err);
		} else {
			console.log('Broadcast results:', body);
			console.log("Transaction send with hash:", tx.getId());
		}
	});
}

// fee to pay the miners
var tx_fee = 10000; 

// prompt for the private key of the source address
prompt('Enter the private key of the source address (WIF format): ', function (private_wif) { 

	// get the source bitcoin address from the private key
	var private_key = bitcoin.ECKey.fromWIF(private_wif);
	var source_address = private_key.pub.getAddress().toString();

	// query blockchain.info for the unspent outputs from the source address
	var url = "https://blockchain.info/unspent?active=" + source_address;
	request(url, function (error, response, body) {
		if (!error && response.statusCode == 200) {
			// parse the response and get the first unspent output
			var json = JSON.parse(body);
			var unspent = json["unspent_outputs"][0];

			// prompt for the destination address
			console.log("Found an unspent transaction output with", satoshiToBTC(unspent.value), "BTC.");
			prompt('Enter a destination address: ', function (dest_address) { 

				// calculate the withdraw amount minus the tx fee
				var withdraw_amount = unspent.value - tx_fee;

				// build a transaction 
				var txb = new bitcoin.TransactionBuilder(bitcoin.networks.bitcoin);
				txb.addInput(unspent.tx_hash_big_endian, unspent.tx_output_n);
				txb.addOutput(dest_address, withdraw_amount);
				txb.sign(0, private_key);
				var tx = txb.build();

				// prompt to confirm sending the transaction
				var confirm = "Send " + satoshiToBTC(withdraw_amount) + " plus miner fee? (y/N):";
				prompt(confirm, function(result) {
					if (result.toUpperCase() == "Y") {
						broadcast_tx(tx);
					};
				});
			});
		} else {
			console.log("Unable to find any unspent transaction outputs.");
			if (error)
				console.log("ERROR:", error);
		}
	});
});






			



// KzCDH91ggHytYbHpDu6b1aTxtGFKCYEFUHrmkGVNcFnZC1uTpPpX
// 1M13pL7E9ZvmFo1PqXHFzR5aWeeMmbzz69

// L5CZenc57iYUwF7P5A7F5tqjrXSpoufvMe6wSkUgtESuyTiZohcW
// 1BQGrTbpp7RnPrSVMXbQ1j9kCf71DZBtfC

// L1pZp59uYSmCKpfEzuRmciNpre4sV1aY9eiSE9JHGPMjDTmA1SyQ
// 14egKB8ybvRupSAgsA17Z5Gr17yadaHuKA

// node send-transaction.js 
// Transction Hash:  3ab285d6f12b8829150e0ee0309e627688aa89fa3398e395ebea8dabe31c37a0
